# VolumeLock copyright (c) 2004-2008 by Peter Watkins (peterw@tux.org) 
#
# This code is derived from code with the following copyright message
# and released under the same license terms:
#
# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.

use strict;

package Plugins::VolumeLock::Plugin;

use Slim::Control::Request;
use Slim::Utils::Strings qw (string);
use Slim::Utils::Prefs;

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.VolumeLock',
#	'defaultLevel' => 'WARN',
#	'defaultLevel' => 'DEBUG',
#	'description'  => getDisplayName(),
});

my $prefs = preferences('plugin.VolumeLock');
my $serverPrefs = preferences('server');

# migrate 6.x prefs
$prefs->migrateClient(1, sub {
	my ($clientprefs, $client) = @_;
	$clientprefs->set('volumeLockVolume', Slim::Utils::Prefs::OldPrefs->clientGet($client, 'volume'));
	1;
});

use vars qw($VERSION);
$VERSION = &vlVersion();

# whether we need to add our callback function to the stack
my $callbackSet = 0;

# whether we're enabled (the callback remains in the function 
# stack after "disabling" the plugin, so we need to keep track of this)
my $pluginEnabled = 0;

# for 6.5+, need to keep track of original volume functions
my $originalMixerVolumeCommand;
my $originalMixerMutingCommand;

sub initPlugin {
	if ( $callbackSet == 0 ) {
		if (! $::noweb) {
			require Plugins::VolumeLock::Settings;
		}
		my $mode = 'plugin.VolumeLock';
		Slim::Buttons::Common::addMode($mode, getFunctions(), \&setMode);
		Slim::Buttons::Home::addSubMenu('PLUGINS', 'PLUGIN_VOLUME_LOCK',  { 'useMode'  => $mode, });
		# use the cleaner addDispatch technique
		if ( substr($::VERSION,0,3) lt 7.4 ) {
			$originalMixerVolumeCommand = Slim::Control::Request::addDispatch(['mixer', 'volume', '_newvalue'],[1, 0, 0, \&VolumeLock_mixerVolumeCommand]);
			$originalMixerMutingCommand = Slim::Control::Request::addDispatch(['mixer', 'muting', '_newvalue'],[1, 0, 0, \&VolumeLock_mixerMutingCommand]);
		} else {
			$originalMixerVolumeCommand = Slim::Control::Request::addDispatch(['mixer', 'volume', '_newvalue'],[1, 0, 1, \&VolumeLock_mixerVolumeCommand]);
			$originalMixerMutingCommand = Slim::Control::Request::addDispatch(['mixer', 'muting', '_newvalue'],[1, 0, 1, \&VolumeLock_mixerMutingCommand]);
		}
		if (! $::noweb) {
			Plugins::VolumeLock::Settings->new;
		}
		$callbackSet = 1; 
	}
	# note that we should act
	$pluginEnabled = 1;
}

sub shutdownPlugin {
	# we should not act
	$pluginEnabled = 0;
}

sub enabled {
	if ( substr($::VERSION,0,3) gt '7.0' ) {
		$log->warn("This plugin has not been tested with SlimServer newer than 7.0.x\n");
	}
	return ($::VERSION ge '7.0');
}

sub vlVersion() {
	my $rcsVersion = '$Revision: 1.20 $';
	$rcsVersion =~ s/.*:\s*([0-9\.]*).*$/$1/;
	return $rcsVersion;
}

sub getDisplayName() {
	return 'PLUGIN_VOLUME_LOCK';
}

# I think this should probably go in the Settings menu,
# but that does not seem to be possible
#sub addMenu {
#	return "SETTINGS";
#}

my @browseMenuChoices = (
	'PLUGIN_VOLUME_ALLOW_ALL',
	'PLUGIN_VOLUME_FIX',
	'PLUGIN_VOLUME_SET_MAX'
);

sub setMode {
	my $client = shift;
	my $method = shift;

	if ($method eq 'pop') {
		Slim::Buttons::Common::popMode($client);
		return;
	}

	my %params = (
		'listRef'        => [
                                        {
                                                name   => '{PLUGIN_VOLUME_ALLOW_ALL}',
                                                value  => 'PLUGIN_VOLUME_ALLOW_ALL',
                                        },
                                        {
                                                name   => '{PLUGIN_VOLUME_FIX}',
                                                value  => 'PLUGIN_VOLUME_FIX',
                                        },
                                        {
                                                name   => '{PLUGIN_VOLUME_SET_MAX}',
                                                value  => 'PLUGIN_VOLUME_SET_MAX',
                                        },
                                ],
		'externRefArgs'  => 'V',
		'header'         => '{PLUGIN_VOLUME_LOCK}{count}',
		'stringHeader'   => 1,
		'overlayRef'      => sub { return (undef,shift->symbols('rightarrow')) },
		'overlayRefArgs'  => '',
		'onRight'      => sub { 
					my $client = $_[0];
					my $value = $_[1]->{'value'};
					$prefs->client($client)->set('volumeLockMode', $value); 
					$prefs->client($client)->set('volumeLockVolume',$serverPrefs->client($client)->get("volume"));
					},
		'pref'          => \&volumeLockInit,
		'initialValue'  => \&volumeLockInit,
		'init'          => \&volumeLockInit,
	);
		
	Slim::Buttons::Common::pushMode($client, 'INPUT.Choice', \%params);
}

sub volumeLockInit {
	my $client = shift;
	if ( (!defined($prefs->client($client)->get('volumeLockMode'))) ||
	     ( $prefs->client($client)->get('volumeLockMode') !~ m/^PLUGIN/ ) ) {
             	$prefs->client($client)->set('volumeLockMode','PLUGIN_VOLUME_ALLOW_ALL');
	}
	return $prefs->client($client)->get('volumeLockMode');
}

sub setPref {
        my $client = shift;
        my $value = shift;
	# See the hash defines above.
	if (ref $value eq 'HASH') {
		$value = $value->{'value'};
	}
	my $pref = $client->modeParam('pref');
	if ( $pref eq 'PLUGIN_VOLUME_ALLOW_ALL' ) {
		$prefs->client($client)->set('volumeLockVolume',-1);
	} else {
		$prefs->client($client)->set('volumeLockVolume',$serverPrefs->client($client)->get("volume")); 
	}
	$prefs->client($client)->set($pref,$value);
}

sub VolumeLock_mixerVolumeCommand {
	$log->debug("VolumeLock_mixerVolumeCommand running\n");
	my @args = @_;
	my $request = $args[0];
	my $client = $request->client();
 	if ($pluginEnabled == 1) {

 		my $newvalue = $request->getParam('_newvalue');

 		&volumeLockInit($client);

 		my $volumeLockVolume = $prefs->client($client)->get('volumeLockVolume');

 		if ( defined($volumeLockVolume) && 

              	     ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_FIX')  &&
			($newvalue != $volumeLockVolume) ) { 

 			$log->info("this player's volume is locked\n");

			# if 7.4+ and SqueezePlay, reset volume
			if (! ( (substr($::VERSION,0,3) ge 7.4) && $client->isa("Slim::Player::SqueezePlay") ) ) {
 				&indicateLocked($client);

 				$request->setStatusDone;

 				return;
			}
			Slim::Utils::Timers::killTimers($client, \&resetVolume);
 			Slim::Utils::Timers::setTimer($client,Time::HiRes::time() + 0.05,\&resetVolume);
 		} elsif ( defined($volumeLockVolume) &&

              	          ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_SET_MAX') ) { 

 			# what volume are they tring to set?

 			my $currentVolume = $serverPrefs->client($client)->get("volume"); 

 			my $checkIt = 1;

 			if ( $newvalue =~ m/^\+(\d*)$/ ) {

 				$newvalue = $currentVolume + $1;

 			} elsif ( $newvalue =~ m/^\-(\d*)$/ ) {

 				$checkIt = 0;

 			}

 			# if > max volume

 			if ( $checkIt && ($newvalue > $volumeLockVolume) ) {

 				# set to max volume

				if (! ( (substr($::VERSION,0,3) ge 7.4) && $client->isa("Slim::Player::SqueezePlay") ) ) {
 				#Slim::Control::Request::executeRequest($client, ['mixer', 'volume', $volumeLockVolume], undef, undef);

 					# indicate capped

 					&indicateCapped($client);

 					$request->setStatusDone;

					$client->execute(['mixer', 'volume', $volumeLockVolume]);
					return;
				} else {
					Slim::Utils::Timers::killTimers($client, \&resetVolume);
 					Slim::Utils::Timers::setTimer($client,Time::HiRes::time() + 0.05,\&resetVolume);
				}
 			}

		}
 	}

	# else call the original function
	$log->debug("calling original mixer function\n");
	return &$originalMixerVolumeCommand(@args);
}

sub resetVolume {
	my $client = shift;
 	my $currentVolume = $serverPrefs->client($client)->get("volume"); 
 	my $volumeLockVolume = $prefs->client($client)->get('volumeLockVolume');
	if ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_FIX') {
		if ( $currentVolume != $volumeLockVolume ) {
 			&indicateLocked($client);
			$client->execute(['mixer', 'volume', $volumeLockVolume]);
		}
	} elsif ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_SET_MAX') {
		if ( $currentVolume > $volumeLockVolume ) {
 			&indicateCapped($client);
			$client->execute(['mixer', 'volume', $volumeLockVolume]);
		}
	}
}

sub unMute {
	my $client = shift;
	if ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_FIX') {
		my $mute = $serverPrefs->client($client)->get('mute');
		if ( $mute ) {
 			&indicateLocked($client);
			$client->execute(['mixer', 'muting', '0']);
		}
	}
}


sub VolumeLock_mixerMutingCommand {
	$log->debug("VolumeLock_mixerMutingCommand running\n");
	my @args = @_;
	my $request = $args[0];
	my $client = $request->client();
 	if ($pluginEnabled == 1) {

 		&volumeLockInit($client);

 		my $volumeLockVolume = $prefs->client($client)->get('volumeLockVolume');

 		if ( defined($volumeLockVolume) && 

              	     ($prefs->client($client)->get('volumeLockMode') eq 'PLUGIN_VOLUME_FIX') ) { 

 			$log->info("this player's volume is locked\n");

			if (! ( (substr($::VERSION,0,3) ge 7.4) && $client->isa("Slim::Player::SqueezePlay") ) ) {
 				&indicateLocked($client);

 				$request->setStatusDone;

 				return;

			}
			Slim::Utils::Timers::killTimers($client, \&unMute);
 			Slim::Utils::Timers::setTimer($client,Time::HiRes::time() + 0.05,\&unMute);
 		}

	}
	# else call the original function
	$log->debug("calling original mixer muting function\n");
	return &$originalMixerMutingCommand(@args);
}

sub indicateCapped($) {

	my $client = shift;

 	# tell the user that the volume is capped

 	my ($line1, $line2) = (string('PLUGIN_VOLUME_LOCK'), string('PLUGIN_VOLUME_IS_CAPPED'));

 	$log->info("volume is capped\n");

	$client->showBriefly( {'line' => [$line1, $line2] }, { 'duration' => 1, 'block' => 1, });
}


sub indicateLocked($) {
	my $client = shift;
	# tell the user that the volume is locked
	my ($line1, $line2) = (string('PLUGIN_VOLUME_LOCK'), string('PLUGIN_VOLUME_IS_LOCKED'));
	$log->info("volume is locked\n");
	$client->showBriefly( {'line' => [$line1, $line2] }, { 'duration' => 1, 'block' => 1, });
}

my %functions = (
);

sub lines {
}

sub getFunctions() {
	return \%functions;
}

1;
